/*===========================================================================*
 * PBar.c                                                                    *
 *                                                                           *
 *      Procedures to create or update Progressive Bar                       *
 *                                                                           *
 * EXPORTED PROCEDURES:                                                      *
 *      void show_PBar (char* label_string)                                  *
 *      gboolean update_PBar(gfloat ratio)                                   *
 *                                                            (by A. Miyake) *
 *===========================================================================*/

#include  <gtk/gtk.h>
#include "widget_set.h"

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif
#include "intl.h"

/*==================*
 * STATIC VARIABLES *
 *==================*/
static GtkWidget* PBar;
static gboolean PBarStat;

/*====================*
 * Internal Procedure *
 *====================*/
static gboolean PBar_delete(GtkObject *object, gpointer data);
static void PBar_cancel(GtkButton *button, gpointer data);

/*===========================================================================*
 * (Exported Function)                                                       *
 *                    show_PBar                                              *
 *===========================================================================*/
void
show_PBar (char* label_string)
{
  GtkWidget* win = gtk_dialog_new ();
  GtkWidget* box = GTK_DIALOG(win)->vbox;
  GtkWidget* ActionBox = GTK_DIALOG (win)->action_area; 
  GtkWidget* label = gtk_label_new(label_string);
  GtkAdjustment* adj = (GtkAdjustment *)gtk_adjustment_new(0, 0, 400, 0, 0, 0);
  GtkWidget* cancel;
  
  gtk_container_set_border_width (GTK_CONTAINER (win), 10);
  gtk_box_pack_start(GTK_BOX(box), label, FALSE, FALSE, 5);
  PBar = gtk_progress_bar_new_with_adjustment(adj);
  gtk_box_pack_start(GTK_BOX(box), PBar, FALSE, FALSE, 5);
  PBarStat = TRUE;	 

  cancel = pack_button(ActionBox, _("Cancel"));

  gtk_widget_show_all(win);
  gtk_grab_add(GTK_WIDGET(win));

  gtk_signal_connect(GTK_OBJECT(win), "delete_event",
		     GTK_SIGNAL_FUNC(PBar_delete), NULL);
  gtk_signal_connect(GTK_OBJECT(cancel), "clicked",
		     GTK_SIGNAL_FUNC(PBar_cancel), NULL);

}

/*===========================================================================*
 * (Exported Function)                                                       *
 *                    show_PBar                                              *
 *===========================================================================*/
gboolean
update_PBar(gfloat ratio)
{
   while (gtk_events_pending ())
     gtk_main_iteration ();

   if(PBarStat == FALSE)
     return FALSE;
      
   gtk_progress_set_percentage(GTK_PROGRESS(PBar), ratio);
   if(ratio == 1.0)
     gtk_widget_destroy(gtk_widget_get_toplevel(GTK_WIDGET(PBar)));

   return TRUE;
}    


/*===========================================================================*
 * (Exported Function)                                                       *
 *                    PBar_delete                                            *
 *                    PBar_cancel                                            *
 *===========================================================================*/
static gboolean 
PBar_delete(GtkObject *object, gpointer data)
{
   PBarStat = FALSE;
   return FALSE;
}

static void
PBar_cancel (GtkButton *button, gpointer data)
{
   PBarStat = FALSE;
   gtk_widget_destroy (gtk_widget_get_toplevel(GTK_WIDGET(button)));
}

   
