/*===========================================================================*
 * get_all_rgb_data.c                                                        *
 *                                                                           *
 *      Procedures for obtaining all conoscopic RGB DATA                     *
 *                                                                           *
 * EXPORTED PROCEDURES:                                                      *
 *      gboolean get_all_RGB(guchar** RGB, gfloat *bright0, gfloat gamma0)   *
 *                                                            (by A. Miyake) *
 *===========================================================================*/
#include  <gtk/gtk.h>
#include <math.h>
#include "cono_map.h"
#include "PBar.h"


#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif
#include "intl.h"


/*==================*
 * GLOBAL VARIABLES *
 *==================*/
extern gint image_size;
extern gint total_image_size;
extern gint image_num;

/*==================*
 * GLOBAL FUNCTIONS *
 *==================*/
extern void get_nth_UniaxialAxis(gint n); 
extern void get_nth_BiaxialAxes(gint n); 
extern gfloat get_conoscopic_RGB(float*);

/*==================*
 * STATIC FUNCTIONS *
 *==================*/
static void get_CircularRGB(guchar** RGB);

/*===========================================================================*
 * (EXPORTED PROCEDURE)                                                      *
 *                         get_all_RGB                                       *
 *===========================================================================*/
gboolean 
get_all_RGB(guchar** RGB, gfloat *bright0, gfloat gamma0)
{
   gint n, k;
   gfloat *fRGB;
   guchar *RGB0;
   gfloat *max_fRGB, most_max;
   gfloat pow_factor;

   show_PBar(_("Calculating Now..."));
   
   fRGB = (gfloat *)g_malloc(sizeof(gfloat)*3*image_size*image_size);
   max_fRGB = (gfloat *)g_malloc(sizeof(gfloat)*image_num);

   most_max = 0;
   for( n = 0; n < image_num; n++)
   {
      if(drawing_method.axial_type == Uniaxial)
	get_nth_UniaxialAxis(n);
      else
	get_nth_BiaxialAxes(n);
      
      max_fRGB[n] = get_conoscopic_RGB(fRGB);
      if(max_fRGB[n] > most_max) most_max = max_fRGB[n];
   
      RGB0 = *RGB + 3*image_size*image_size*n;
      for(k = 0; k < 3*image_size*image_size; k++)
      {
	 if(fRGB[k] < 0) 
	   RGB0[k] = 255;
	 else
	   RGB0[k] = rint(254 * pow(fRGB[k]/max_fRGB[n], 1/gamma0));
      }

      if(update_PBar((float)(n+1)/image_num) == FALSE)  
      {
	 g_free(fRGB); g_free(max_fRGB);
	 return FALSE;
      }
   }

   for(n = 0; n < image_num; n++)
   {
      pow_factor = pow(max_fRGB[n]/most_max, 1/gamma0);
      RGB0 = *RGB + 3*image_size*image_size*n;

      for(k = 0; k < 3*image_size*image_size; k++)
      {
     	 if(RGB0[k] < 254.5)
	   RGB0[k] = rint(RGB0[k] * pow_factor);
      }
   }

   if(drawing_method.drawing_kind == Circular)
     get_CircularRGB(RGB);

   *bright0 = 1.0/most_max;

   g_free(fRGB);  g_free(max_fRGB);
   
   return TRUE;
}
   
/*===========================================================================*
 * (STATIC PROCEDURE)                                                        *
 *                         get_CircularRGB                                   *
 *===========================================================================*/
void 
get_CircularRGB(guchar** RGB)
{
   gint n;
   gint k, destk;
   gint startx, starty, x, y;
   gdouble ph;
   guchar *RGBCirc, *RGB0; 
   gdouble R, r;
 
   RGBCirc
    = (guchar *)g_malloc(3*sizeof(guchar)*total_image_size*total_image_size);

   for(k = 0; k < 3*total_image_size*total_image_size; k++)
     RGBCirc[k] = 255;

   for( n = 0; n < image_num; n++ )
   {
      RGB0 = *RGB + 3*n*image_size*image_size;

      ph = n*360.0/image_num;
      R = (total_image_size-image_size)/2;
      r = image_size/2;
      startx = rint(R*cos(ph*M_PI/180) + R);
      starty = rint(-R*sin(ph*M_PI/180) + R);
      for(k = 0; k < image_size*image_size; k++)
      {
	 x = k%image_size;
	 y = k/image_size;
	 destk = x + startx + total_image_size*(y+starty);
	
	 if(RGB0[3*k] != 255 || RGB0[3*k+1] != 255 || RGB0[3*k+2] != 255)
	 { 
	    RGBCirc[3*destk] = RGB0[3*k];
	    RGBCirc[3*destk+1] = RGB0[3*k+1];
	    RGBCirc[3*destk+2] = RGB0[3*k+2];
	 }
      }
   }    

   g_free(*RGB);
   *RGB = RGBCirc;
   image_num = 1;
   image_size = total_image_size;
}


